{ *************************************************************************** }
{                                                                             }
{ Hotzonu 2 BE Board Dialog Unit                                              }
{                                                                             }
{ 2001-05 Hotzonu Project, Monazilla Software Foundation 2001                 }
{                                                                             }
{ *************************************************************************** }

unit Be2chBoard;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, EnhListView, ExtListView, Buttons, IniFiles, ZonuClass,
  Menus, StdCtrls;

type
  TBe2chBoardDialog = class(TForm)
    ListView1: TdfsExtListView;
    ListView2: TdfsExtListView;
    SpeedButton1: TSpeedButton;
    SpeedButton2: TSpeedButton;
    PopupMenu1: TPopupMenu;
    CharsetMenu: TMenuItem;
    Button6: TButton;
    Button7: TButton;
    procedure FormCreate(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure SpeedButton2Click(Sender: TObject);
    procedure Button6Click(Sender: TObject);
    procedure Button7Click(Sender: TObject);
  private
    { Private 錾 }
    FSite: TSite;
    procedure BuildBeList;
    procedure AddListItem(ListView: TdfsExtListView; Board: TBoard);
    procedure BuildCharsetMenu;
    procedure ChangeCharset(Sender: TObject);
  public
    { Public 錾 }

  end;

var
  Be2chBoardDialog: TBe2chBoardDialog;

implementation

{$R *.dfm}

uses
  Zonutils, Hzbconst;

{ TBe2chBoardDialog }

procedure TBe2chBoardDialog.FormCreate(Sender: TObject);
begin
  ZonuEnviron.SiteList.FindSite('2ch', FSite);
  BuildBeList;
  BuildCharsetMenu;
end;

procedure TBe2chBoardDialog.BuildBeList;
var
  ini: TMemIniFile;
  i: integer;
begin
  ini   :=  TMemIniFile.Create(FSite.ConfigPath + 'be.ini');
  for i :=  0 to  FSite.BoardList.Count - 1 do  begin
    if  (ini.ReadBool('BE', FSite.BoardList[i].BoardID, False)) then  begin
      AddListItem(ListView2, FSite.BoardList[i]);
    end else begin
      AddListItem(ListView1, FSite.BoardList[i]);
    end;
  end;
  ini.Free;
end;

procedure TBe2chBoardDialog.AddListItem(ListView: TdfsExtListView;
  Board: TBoard);
var
  Item: TListItem;
begin
  Item  :=  ListView.Items.Add;
  Item.Caption  :=  Board.CategoryName;
  Item.SubItems.Add(Board.Name);
  Item.SubItems.Add(CharsetToText(Board.Charset));
  Item.Data :=  Board;
end;

procedure TBe2chBoardDialog.SpeedButton1Click(Sender: TObject);
begin
  if  (ListView1.Selected <> nil) then  begin
    AddListItem(ListView2, TBoard(ListView1.Selected.Data));
    ListView1.Selected.Delete;
  end;
end;

procedure TBe2chBoardDialog.SpeedButton2Click(Sender: TObject);
begin
  if  (ListView2.Selected <> nil) then  begin
    AddListItem(ListView1, TBoard(ListView2.Selected.Data));
    ListView2.Selected.Delete;
  end;
end;


procedure TBe2chBoardDialog.BuildCharsetMenu;
  procedure AddCharsetMenu(Charset: TCharset);
  var
    Item: TMenuItem;
  begin
    Item  :=  TMenuItem.Create(Self);
    CharsetMenu.Add(Item);
    Item.Caption  :=  CharsetToText(Charset);
    Item.OnClick  :=  ChangeCharset;
    Item.Tag      :=  Integer(Charset);
  end;
begin
  AddCharsetMenu(csSJIS);
  AddCharsetMenu(csEUC);
  AddCharsetMenu(csUTF8);
end;

procedure TBe2chBoardDialog.ChangeCharset(Sender: TObject);
begin
  if  (ListView2.Selected <> nil) then  begin
    TBoard(ListView2.Selected.Data).CharSet :=  TCharset(TMenuItem(Sender).Tag);
    ListView2.Selected.SubItems[1] := CharsetToText(TBoard(ListView2.Selected.Data).CharSet);
  end;
end;

procedure TBe2chBoardDialog.Button6Click(Sender: TObject);
var
  ini: TMemIniFile;
  i: integer;
begin
  ini   :=  TMemIniFile.Create(FSite.ConfigPath + 'be.ini');
  ini.EraseSection('BE');
  ini.EraseSection('CHARSET');
  for i :=  0 to  ListView2.Items.Count - 1 do  begin
    ini.WriteBool('BE',
                  TBoard(ListView2.Items[i].Data).BoardID,
                  True);
    if  (TBoard(ListView2.Items[i].Data).CharSet <> FSite.CharSet)  then  begin
      ini.WriteString('CHARSET',
                      TBoard(ListView2.Items[i].Data).BoardID,
                      CharsetToText(TBoard(ListView2.Items[i].Data).Charset));
    end;
  end;
  ini.UpdateFile;
  ini.Free;
  ModalResult :=  mrOk;
end;

procedure TBe2chBoardDialog.Button7Click(Sender: TObject);
begin
  ModalResult :=  mrCancel;
end;

end.
