{ *************************************************************************** }
{                                                                             }
{ Hotzonu 2ch Compatible Script Unit                                          }
{                                                                             }
{ 2001-04 Hotzonu Project, Monazilla Software Foundation 2001                 }
{                                                                             }
{ *************************************************************************** }

unit Zonu2ch;

interface

uses
  Classes, SysUtils, ZonuClass, ZonuCommon, ZonuHttp, Hzbconst, ZonuConf;

type
  PParamData  = ^TParamData;
  TParamData  = record
    //
  end;

  T2chScript = class(TBBS)
  private
    { private }
  protected
    { protected }
    function GetThreadListType: TThreadListAccessType; override;
    function GetThreadDownloadMode: TThreadDownloadMode; override;
    function GetNeedHRefLink: boolean; override;
    function GetNeedLinkTarget: boolean; override;
    function GetRelativeLink: boolean; override;
    function GetIsPublished: boolean; override;
    procedure ZonuDataThreadList(Sender: TObject; Downloader: TDownloader;
      Data: string; TotalSize, DownloadSize: integer; var MoreData: boolean); override;
    procedure ZonuEndThreadList(Sender: TObject; Downloader: TDownloader;
      Error: boolean); override;
    procedure ZonuThreadListError(Sender: TObject; Downloader: TDownloader;
      ErrorCode: integer; ErrorMsg, LastResponse: string); override;
    procedure ZonuDataRes(Sender: TObject; Downloader: TDownloader;
      Data: string; TotalSize, DownloadSize: integer; var MoreData: boolean); override;
    procedure ZonuEndRes(Sender: TObject; Downloader: TDownloader;
      Error: boolean); override;
    procedure ZonuResError(Sender: TObject; Downloader: TDownloader;
      ErrorCode: integer; ErrorMsg, LastResponse: string); override;
  public
    { public }
    procedure ParseSubject(subject: string; out FileName: string;
      out Title: string; out Count: integer); override;
    procedure ParseDat(const ALine: string; const index: integer;
      var ARes: TRes; out ASubject: string); override;
    procedure DownloadThreadList(Downloader: TThreadListDownloader;
      const NoCache, ASync, InfSave: boolean); override;
    procedure DownloadThreadItem(Downloader: TResDownloader;
      const NoCache, ASync, InfSave, AutoPilot: boolean); override;
    procedure SendRes(Uploader: TResUploader); override;
    procedure SendThread(Uploader: TThreadUploader); override;
    function ConvUrl(const Url: string): string; override;
    function  GetDisplayUrl(ThreadItem: TThreadItem): string; override;
  end;

implementation

{ T2chScript }

uses
  MonaUtils;

//
//  Xbhꗗ̎擾@
//
function T2chScript.GetThreadListType: TThreadListAccessType;
begin
  Result  :=  tlLump;
end;

//
//  Xbh̎擾@
//
function T2chScript.GetThreadDownloadMode: TThreadDownloadMode;
begin
  Result  :=  dmText;
end;

//
//  http://ւ̃Nǉ
//
function T2chScript.GetNeedHRefLink: boolean;
begin
  Result  :=  True;
end;

//
//  NւTarget=_blankǉ
//
function T2chScript.GetNeedLinkTarget: boolean;
begin
  Result  :=  False;
end;

//
//  BBSEBU[hŕ\邩ǂ
//
function T2chScript.GetIsPublished: boolean;
begin
  Result  :=  True;
end;

//
//  Xbhꗗ_E[h
//
procedure T2chScript.DownloadThreadList(Downloader: TThreadListDownloader;
  const NoCache, ASync, InfSave: boolean);
var
  ParamData : PParamData;
  FZonuHttp : TZonuHttp;
begin
  New(ParamData);
  Downloader.ParamData      :=  ParamData;
  FZonuHttp                 :=  TZonuHttp.Create(nil);
  FZonuHttp.Downloader      :=  Downloader;
  FZonuHttp.OnZonuData      :=  ZonuDataThreadList;
  FZonuHttp.OnZonuEnd       :=  ZonuEndThreadList;
  FZonuHttp.OnZonuError     :=  ZonuThreadListError;
  FZonuHttp.Url             :=  GetSubjectUrl(Downloader.Board);
  FZonuHttp.NoCache         :=  NoCache;
  FZonuHttp.UserAgent       :=  USER_AGENT;
  FZonuHttp.InfSave         :=  InfSave;
  FZonuHttp.Username        :=  Downloader.Board.BasicAuthorization_UserName; //8
  FZonuHttp.Password        :=  Downloader.Board.BasicAuthorization_Password; //8
  FZonuHttp.GetModified(Downloader.Board.CachePath + 'subject.inf', ASync);
end;

//
//  Xbhꗗ擾
//
procedure T2chScript.ZonuDataThreadList(Sender: TObject;
  Downloader: TDownloader; Data: string; TotalSize,
  DownloadSize: integer; var MoreData: boolean);
var
  dl        : TThreadListDownloader;
  AFileName : string;
  ATitle    : string;
  ACount    : integer;
  ThreadItem: TThreadItem;
begin
  inherited;
  dl  :=  TThreadListDownloader(Downloader);
  ParseSubject(Data, AFileName, ATitle, ACount);
  ThreadItem                    :=  TThreadItem.Create;
  ThreadItem.BBS                :=  Self;
  ThreadItem.Site               :=  dl.Site;
  ThreadItem.BoardID            :=  dl.Board.BoardID;
  ThreadItem.No                 :=  0;
  ThreadItem.DatFileName        :=  AFileName;
  ThreadItem.Title              :=  ATitle;
  ThreadItem.AgeCount           :=  ACount;
  ThreadItem.RemoteDatFileName  :=  GetDatUrl(dl.Board, AFileName);
  ThreadItem.LocalDatFileName   :=  dl.Board.CachePath + AFileName;
  dl.FireDataEvt(ThreadItem, DownloadSize, TotalSize, MoreData);
end;

//
//  Xbhꗗ擾I
//
procedure T2chScript.ZonuEndThreadList(Sender: TObject;
  Downloader: TDownloader; Error: boolean);
var
  ParamData:  PParamData;
begin
  ParamData :=  PParamData(Downloader.ParamData);
  Dispose(ParamData);
  inherited;
end;

//
//  Xbhꗗ擾G[
//
procedure T2chScript.ZonuThreadListError(Sender: TObject;
  Downloader: TDownloader; ErrorCode: integer; ErrorMsg, LastResponse: string);
begin
  inherited;

end;

//
//  Xbhꗗ̉
//
procedure T2chScript.ParseSubject(subject: string; out FileName,
  Title: string; out Count: integer);
begin
  inherited;
  Parse2chSubject(subject, FileName, Title, Count);
end;

//
//  X̉
//
procedure T2chScript.ParseDat(const ALine: string; const index: integer;
  var ARes: TRes; out ASubject: string);
begin
  inherited;
  Parse2chDat(ALine, Index, ARes, ASubject);
end;

//
//  \URL
//
function T2chScript.GetDisplayUrl(ThreadItem: TThreadItem): string;
begin
  if  (ZonuConfig.ReadMode = rmLast50)  then  begin
    Result  :=  GetReadCgiUrl(ThreadItem) + '&ls=50';
  end else begin
    Result  :=  GetReadCgiUrl(ThreadItem);
  end;
end;

//
//  Xbh̃_E[h
//
procedure T2chScript.DownloadThreadItem(Downloader: TResDownloader;
  const NoCache, ASync, InfSave, AutoPilot: boolean);
var
  ParamData : PParamData;
  FZonuHttp : TZonuHttp;
begin
  New(ParamData);
  Downloader.ParamData      :=  ParamData;
  FZonuHttp                 :=  TZonuHttp.Create(nil);
  FZonuHttp.Downloader      :=  Downloader;
  FZonuHttp.OnZonuData      :=  ZonuDataRes;
  FZonuHttp.OnZonuEnd       :=  ZonuEndRes;
  FZonuHttp.OnZonuError     :=  ZonuResError;
  FZonuHttp.Url             :=  Downloader.ThreadItem.RemoteDatFileName;
  FZonuHttp.NoCache         :=  NoCache;
  FZonuHttp.UserAgent       :=  USER_AGENT;
  FZonuHttp.NeedDataEvent   :=  True;
  FZonuHttp.CacheReady      :=  False;
  FZonuHttp.InfSave         :=  InfSave;
  FZonuHttp.Username        :=  Downloader.Board.BasicAuthorization_UserName; //8
  FZonuHttp.Password        :=  Downloader.Board.BasicAuthorization_Password; //8

  FZonuHttp.CacheReady      :=  Downloader.CacheReady;
  Downloader.HttpObj        :=  FZonuHttp;
  FZonuHttp.GetTextDiff(ChangeFileExt(Downloader.ThreadItem.LocalDatFileName, '.inf'), ASync);
end;

//
//  X擾
//
procedure T2chScript.ZonuDataRes(Sender: TObject; Downloader: TDownloader;
  Data: string; TotalSize, DownloadSize: integer; var MoreData: boolean);
var
  dl  : TResDownloader;
  ASubject : string;
  Res : TRes;
begin
  dl  :=  TResDownloader(Downloader);
  Res :=  TRes.Create;
  ParseDat(Data, 0, Res, ASubject);
  Res.No          :=  dl.ResList.Count + 1;
  Res.BBS         :=  dl.BBS;
  Res.Site        :=  dl.Site;
  Res.ThreadTitle :=  ASubject;
  if  (dl.ThreadItem.Title = '')  then  begin
    if  (ASubject <> '')  then  begin
      dl.ThreadItem.Title :=  ASubject;
    end;
  end;
  dl.FireDataEvt(Res, DownloadSize, TotalSize);
end;

//
//  X擾I
//
procedure T2chScript.ZonuEndRes(Sender: TObject; Downloader: TDownloader;
  Error: boolean);
var
  ParamData:  PParamData;
begin
  ParamData :=  PParamData(Downloader.ParamData);
  Dispose(ParamData);
  inherited;
end;

//
//  X擾G[
//
procedure T2chScript.ZonuResError(Sender: TObject; Downloader: TDownloader;
  ErrorCode: integer; ErrorMsg, LastResponse: string);
begin
  inherited;

end;

//
//  
//
procedure T2chScript.SendRes(Uploader: TResUploader);
var
  FZonuHttp : TZonuHttp;
begin
  FZonuHttp                 :=  TZonuHttp.Create(nil);
  FZonuHttp.Uploader        :=  Uploader;
  FZonuHttp.OnZonuSendEnd   :=  ZonuSendEnd;
  FZonuHttp.OnZonuSendError :=  ZonuSendError;
  FZonuHttp.Url             :=  GetSendUrl(Uploader.Board, Uploader.ThreadItem);
  FZonuHttp.UserAgent       :=  USER_AGENT;
  FZonuHttp.SendReferer     :=  Uploader.BBS.GetReadCgiUrl(Uploader.ThreadItem);
  FZonuHttp.SendContent     :=  Uploader.SendData.ConvSendContent(GetSendParam);
  FZonuHttp.SendCookie      :=  Uploader.SendData.ConvSendContent(GetSendCookie);
  FZonuHttp.Username        :=  Uploader.Board.BasicAuthorization_UserName; //8
  FZonuHttp.Password        :=  Uploader.Board.BasicAuthorization_Password; //8
  FZonuHttp.Send;
end;

//
//  VX
//
procedure T2chScript.SendThread(Uploader: TThreadUploader);
var
  FZonuHttp   : TZonuHttp;
begin
  FZonuHttp                 :=  TZonuHttp.Create(nil);
  FZonuHttp.Uploader        :=  Uploader;
  FZonuHttp.OnZonuSendEnd   :=  ZonuSendEnd;
  FZonuHttp.OnZonuSendError :=  ZonuSendError;
  FZonuHttp.Url             :=  GetNewThreadUrl(Uploader.Board);
  FZonuHttp.UserAgent       :=  USER_AGENT;
  FZonuHttp.SendReferer     :=  Uploader.Board.Url;
  FZonuHttp.SendContent     :=  Uploader.SendData.ConvSendContent(GetNewThreadParam);
  FZonuHttp.SendCookie      :=  Uploader.SendData.ConvSendContent(GetNewThreadCookie);
  FZonuHttp.Username        :=  Uploader.Board.BasicAuthorization_UserName; //8
  FZonuHttp.Password        :=  Uploader.Board.BasicAuthorization_Password; //8
  FZonuHttp.Send;
end;

function T2chScript.GetRelativeLink: boolean;
begin
  Result  :=  True;
end;

function T2chScript.ConvUrl(const Url: string): string;
begin
  Result  :=  Conv2chUrl(Url);
end;

end.
