{ *************************************************************************** }
{                                                                             }
{ Hotzonu DOLIB/BE Script Unit                                                }
{                                                                             }
{ 2001-05 Hotzonu Project, Monazilla Software Foundation 2001                 }
{                                                                             }
{ *************************************************************************** }

unit ZonuDolib;

interface

uses
  Classes, SysUtils, ZonuClass, ZonuCommon, ZonuHttp, Hzbconst, HttpProt, Forms,
  Dialogs, Controls, IniFiles;

type
  PParamData  = ^TParamData;
  TParamData  = record
    Check01: boolean; //
    //FindKakoLogLevel: integer;  //FindKakoLog̃lXg
  end;

  TDolibScript = class(TBBS)
  private
    { private }
    FError: boolean;
    FLog: TStringList;
    procedure WriteLog(const str: string);
  protected
    { protected }
    function GetThreadListType: TThreadListAccessType; override;
    function GetThreadDownloadMode: TThreadDownloadMode; override;
    function GetNeedHRefLink: boolean; override;
    function GetNeedLinkTarget: boolean; override;
    function GetRelativeLink: boolean; override;
    function GetIsPublished: boolean; override;

    function FindKakoLog(Downloader: TResDownloader; const Url: string): boolean;
    function ExecDatDownload(Downloader: TResDownloader;
      const DatUrl: string; const IsGz, KakoLog: boolean): boolean;
    function ExecOfflaw(Downloader: TResDownloader; const Url: string = ''): boolean;
    procedure EndOfflaw(Sender: TObject; Downloader: TDownloader;
      Error: boolean);
    procedure ExpandDat(ms: TMemoryStream; out Content: string; out Size,
      MaxSize: integer; out State: string);

    procedure ZonuSendEnd(Sender: TObject; Uploader: TUploader;
      RecvStr: string; Error: boolean); override;

    procedure ZonuDataThreadList(Sender: TObject; Downloader: TDownloader;
      Data: string; TotalSize, DownloadSize: integer; var MoreData: boolean); override;
    procedure ZonuEndThreadList(Sender: TObject; Downloader: TDownloader;
      Error: boolean); override;
    procedure ZonuThreadListError(Sender: TObject; Downloader: TDownloader;
      ErrorCode: integer; ErrorMsg, LastResponse: string); override;
    procedure ZonuDataRes(Sender: TObject; Downloader: TDownloader;
      Data: string; TotalSize, DownloadSize: integer; var MoreData: boolean); override;
    procedure ZonuEndRes(Sender: TObject; Downloader: TDownloader;
      Error: boolean); override;
    procedure ZonuResError(Sender: TObject; Downloader: TDownloader;
      ErrorCode: integer; ErrorMsg, LastResponse: string); override;
    procedure ZonuNotFound(Sender: TObject; Downloader: TDownloader;
      ErrorCode: integer; Recvstr: string);
  public
    { public }
    constructor Create(const ABBSID: string); override;
    destructor  Destory; 
    procedure ParseSubject(subject: string; out FileName: string;
      out Title: string; out Count: integer); override;
    procedure ParseDat(const ALine: string; const index: integer;
      var ARes: TRes; out ASubject: string); override;
    procedure DownloadThreadList(Downloader: TThreadListDownloader;
      const NoCache, ASync, InfSave: boolean); override;
    procedure DownloadThreadItem(Downloader: TResDownloader;
      const NoCache, ASync, InfSave, AutoPilot: boolean); override;
    procedure SendRes(Uploader: TResUploader); override;
    procedure SendRes2(Uploader: TResUploader);
    procedure SendThread(Uploader: TThreadUploader); override;
    procedure SendThread2(Uploader: TThreadUploader);
    function  GetDisplayUrl(ThreadItem: TThreadItem): string; override;
    function ConvUrl(const Url: string): string; override;
    function  GetReadCgiUrl(ThreadItem: TThreadItem):string; override;
    function  GetResUrl(ThreadItem: TThreadItem; const StartNo, ToNo: integer;
      const NoFirst: boolean = True): string; override;
  end;

implementation

uses
  DolibModule, Be2chModule, Zonutils, ZonuConf, MonaUtils, Windows;

const
  READCGI_ERR     = '-ERR';
  READCGI_INCR    = '-INCR';
  READCGI_OK      = '+OK';
  READCGI_PARTIAL = '+PARTIAL';
  READCGI_ERR_FOUND_KAKO  = '-ERR ߋOqɂŔ';
  READCGI_ERR_NOT_HTML    = '-ERR html҂';
  READCGI_ERR_NOT_FOUND   = '-ERR ȔorXbhȂłB';
  READCGI_ERR_ABONE       = '-ERR ǂłځ[񂪂݂łB';
  READCGI_ERR_TIMEOUT     = '-ERR w莞Ԃ߂܂B';
  READCGI_ERR_CANTUSE     = '-ERR @܂';
  BBSCGI_ERR_SERVER_BUSY  = 'T[ôׂ̕ŏ߂܂B';
  BE2CH_WRITE_ERR         = 'OCG[ 0';             

  RES2CH_TRUE       = '<!-- 2ch_X:true -->';
  RES2CH_FALSE      = '<!-- 2ch_X:false -->';
  RES2CH_ERROR      = '<!-- 2ch_X:error -->';
  RES2CH_CHECK      = '<!-- 2ch_X:check -->';
  RES2CH_COOKIE     = '<!-- 2ch_X:cookie -->';

  LOCATION_OOSUGI   = 'http://www2.2ch.net/live.html';

{ TDolibScript }

//
//  Xbhꗗ̎擾@
//
function TDolibScript.GetThreadListType: TThreadListAccessType;
begin
  Result  :=  tlLump;
end;

//
//  Xbh̎擾@
//
function TDolibScript.GetThreadDownloadMode: TThreadDownloadMode;
begin
  Result  :=  dmText;
end;

//
//  http://ւ̃Nǉ
//
function TDolibScript.GetNeedHRefLink: boolean;
begin
  Result  :=  True;
end;

//
//  NւTarget=_blankǉ
//
function TDolibScript.GetNeedLinkTarget: boolean;
begin
  Result  :=  False;
end;

//
//  BBSEBU[hŕ\邩ǂ
//
function TDolibScript.GetIsPublished: boolean;
begin
  Result  :=  True;
end;

//
//  Xbhꗗ_E[h
//
procedure TDolibScript.DownloadThreadList(Downloader: TThreadListDownloader;
  const NoCache, ASync, InfSave: boolean);
var
  ParamData : PParamData;
  FZonuHttp : TZonuHttp;
begin
  New(ParamData);
  Downloader.ParamData      :=  ParamData;
  FZonuHttp                 :=  TZonuHttp.Create(nil);
  FZonuHttp.Downloader      :=  Downloader;
  FZonuHttp.OnZonuData      :=  ZonuDataThreadList;
  FZonuHttp.OnZonuEnd       :=  ZonuEndThreadList;
  FZonuHttp.OnZonuError     :=  ZonuThreadListError;
  FZonuHttp.Url             :=  GetSubjectUrl(Downloader.Board);
  FZonuHttp.NoCache         :=  NoCache;
  FZonuHttp.UserAgent       :=  DolibManager.UserAgent;
  FZonuHttp.NeedDataEvent   :=  False;
  FZonuHttp.AcceptEncoding  :=  'gzip';
  FZonuHttp.InfSave         :=  InfSave;
  FZonuHttp.Username        :=  Downloader.Board.BasicAuthorization_UserName; //8
  FZonuHttp.Password        :=  Downloader.Board.BasicAuthorization_Password; //8
  FZonuHttp.GetModified(Downloader.Board.CachePath + 'subject.inf', ASync);
end;

//
//  Xbhꗗ擾
//
procedure TDolibScript.ZonuDataThreadList(Sender: TObject;
  Downloader: TDownloader; Data: string; TotalSize,
  DownloadSize: integer; var MoreData: boolean);
var
  dl        : TThreadListDownloader;
begin
  //.gzȂ
  dl  :=  TThreadListDownloader(Downloader);
  dl.FireDataEvt(nil, DownloadSize, TotalSize, MoreData);
end;

//
//  Xbhꗗ擾I
//
procedure TDolibScript.ZonuEndThreadList(Sender: TObject;
  Downloader: TDownloader; Error: boolean);
var
  dl        : TThreadListDownloader;
  AFileName : string;
  ATitle    : string;
  ACount    : integer;
  ThreadItem: TThreadItem;
  sl        : TStringList;
  MoreData  : boolean;
  i   : integer;
  ParamData:  PParamData;
begin
  if  (not Error) then  begin
    sl  :=  TStringList.Create;
    sl.Text   :=  TZonuHttp(Sender).Content;
    for i :=  0 to  sl.Count - 1  do  begin
      dl  :=  TThreadListDownloader(Downloader);
      ParseSubject(sl[i], AFileName, ATitle, ACount);
      ThreadItem                    :=  TThreadItem.Create;
      ThreadItem.BBS                :=  Self;
      ThreadItem.Site               :=  dl.Site;
      ThreadItem.BoardID            :=  dl.Board.BoardID;
      ThreadItem.No                 :=  0;
      ThreadItem.DatFileName        :=  AFileName;
      ThreadItem.Title              :=  ATitle;
      ThreadItem.AgeCount           :=  ACount;
      ThreadItem.RemoteDatFileName  :=  GetDatUrl(dl.Board, AFileName);
      ThreadItem.LocalDatFileName   :=  dl.Board.CachePath + AFileName;
      dl.FireDataEvt(ThreadItem, 0, 0, MoreData);
      Application.ProcessMessages;
    end;
    sl.Free;
  end;
  ParamData :=  PParamData(Downloader.ParamData);
  Dispose(ParamData);
  inherited;
end;

//
//  Xbhꗗ擾G[
//
procedure TDolibScript.ZonuThreadListError(Sender: TObject;
  Downloader: TDownloader; ErrorCode: integer; ErrorMsg, LastResponse: string);
begin
  inherited;
  {
  if  (ErrorCode = 302) and
      (AnsiPos('http://www2.2ch.net/403/', LastResponse) > 0) then
  begin
    //o[{nEX



  end;
  }
end;

//
//  Xbhꗗ̉
//
procedure TDolibScript.ParseSubject(subject: string; out FileName,
  Title: string; out Count: integer);
begin
  inherited;
  Parse2chSubject(subject, FileName, Title, Count);
end;

//
//  X̉
//
procedure TDolibScript.ParseDat(const ALine: string; const index: integer;
  var ARes: TRes; out ASubject: string);
begin
  inherited;
  Parse2chDat(ALine, Index, ARes, ASubject);
end;


//
//  \URL
//
function TDolibScript.GetDisplayUrl(ThreadItem: TThreadItem): string;
begin
  if  (ZonuConfig.ReadMode = rmLast50)  then  begin
    Result  :=  GetReadCgiUrl(ThreadItem) + 'l50';
  end else begin
    Result  :=  GetReadCgiUrl(ThreadItem);
  end;
end;

//
//  Xbh̃_E[h
//
procedure TDolibScript.DownloadThreadItem(Downloader: TResDownloader;
  const NoCache, ASync, InfSave, AutoPilot: boolean);
var
  ParamData : PParamData;
  FZonuHttp : TZonuHttp;
begin
  New(ParamData);
  ParamData.Check01         :=  False;  //NotFoundɎgp
  //ParamData.FindKakoLogLevel:=  0;  
  Downloader.ParamData      :=  ParamData;
  FZonuHttp                 :=  TZonuHttp.Create(nil);
  FZonuHttp.Downloader      :=  Downloader;
  FZonuHttp.OnZonuData      :=  ZonuDataRes;
  FZonuHttp.OnZonuEnd       :=  ZonuEndRes;
  FZonuHttp.OnZonuError     :=  ZonuResError;
  FZonuHttp.OnZonuNotFound  :=  ZonuNotFound;
  FZonuHttp.Url             :=  Downloader.ThreadItem.RemoteDatFileName;
  FZonuHttp.NoCache         :=  NoCache;
  FZonuHttp.UserAgent       :=  DolibManager.UserAgent;
  FZonuHttp.NeedDataEvent   :=  True;
  FZonuHttp.CacheReady      :=  False;
  FZonuHttp.Username        :=  Downloader.Board.BasicAuthorization_UserName; //8
  FZonuHttp.Password        :=  Downloader.Board.BasicAuthorization_Password; //8
  FZonuHttp.InfSave         :=  InfSave;
  FZonuHttp.CacheReady      :=  Downloader.CacheReady;
  Downloader.HttpObj        :=  FZonuHttp;
  FZonuHttp.GetTextDiff(ChangeFileExt(Downloader.ThreadItem.LocalDatFileName, '.inf'), ASync);
end;

//
//  X擾
//
procedure TDolibScript.ZonuDataRes(Sender: TObject;
  Downloader: TDownloader; Data: string; TotalSize, DownloadSize: integer;
  var MoreData: boolean);
var
  dl  : TResDownloader;
  ASubject : string;
  Res : TRes;
begin
  dl  :=  TResDownloader(Downloader);
  Res :=  TRes.Create;
  ParseDat(Data, dl.ResList.Count, Res, ASubject);
  Res.No          :=  dl.ResList.Count + 1;
  Res.BBS         :=  dl.BBS;
  Res.Site        :=  dl.Site;
  Res.ThreadTitle :=  ASubject;
  if  (dl.ThreadItem.Title = '')  then  begin
    if  (ASubject <> '')  then  begin
      dl.ThreadItem.Title :=  ASubject;
    end;
  end;
  dl.FireDataEvt(Res, DownloadSize, TotalSize);
end;

//
//  X擾I
//
procedure TDolibScript.ZonuEndRes(Sender: TObject; Downloader: TDownloader;
  Error: boolean);
var
  ParamData:  PParamData;
begin
  ParamData :=  PParamData(Downloader.ParamData);
  Dispose(ParamData);
  inherited;
end;

//
//  X擾G[
//
procedure TDolibScript.ZonuResError(Sender: TObject;
  Downloader: TDownloader; ErrorCode: integer; ErrorMsg, LastResponse: string);
begin
  inherited;
  {$IFDEF LOG}
    WriteLog('** ERROR **');
    WriteLog(Format('CODE   = %d', [ErrorCode]));
    WriteLog(Format('MSG    = %s', [ErrorMsg]));
    WriteLog(Format('URL    = %s', [TZonuHttp(Sender).Url]));
    if  (DolibManager.Connected)  then  begin
      WriteLog('DOLIB  = CONNECTED');
    end else begin
      WriteLog('DOLIB  = NOT CONNECTED');
    end;
    WriteLog(' ');
  {$ENDIF}
end;

//
//  URL PATHINFO^^ɕϊ
//
function TDolibScript.ConvUrl(const Url: string): string;
begin
  Result  :=  Conv2chUrl(Url);
end;

procedure TDolibScript.ZonuNotFound(Sender: TObject; Downloader: TDownloader;
  ErrorCode: integer; Recvstr: string);
var
  ThreadItem  : TThreadItem;
  dl  : TResDownloader;
  ParamData : PParamData;
  s : string;
begin
  ParamData :=  PParamData(Downloader.ParamData);
  dl  :=  TResDownloader(Downloader);
  if  (not dl.AutoPilot) then  begin
    if  (ErrorCode =  404) or (ErrorCode =  403) or (ErrorCode = 302) then  begin
      if  (ErrorCode = 302) and (AnsiPos('http://www2.2ch.net/403/', Recvstr) > 0) then begin
        dl.FireDownloadError(ErrorCode, '', Recvstr);
        Dispose(ParamData);
        dl.FireEndDownload(True);
      end else begin
        if  (dl.OriginalUrl <> '') and
            (ExtractHostName(TZonuHttp(Sender).Url) <> ExtractHostName(dl.OriginalUrl)) and
            (not ParamData.Check01) then
        begin
          s :=  StringReplace(TZonuHttp(Sender).Url,
                              ExtractHostName(TZonuHttp(Sender).Url),
                              ExtractHostName(dl.OriginalUrl), [rfIgnoreCase]);
          if  (not ExecDatDownload(dl, s, False, False)) then  begin
            ParamData.Check01 :=  True;
            ZonuNotFound(Sender, Downloader, ErrorCode, Recvstr);
            Exit;
          end;
        end else begin
          if  (MessageDlg('Xbh܂łB' + #13#10 +
                          'T[o[ړ]AߋO֗܂B' + #13#10 +
                          'ߋOT܂H', mtWarning,
                          [mbYes, mbNo], 0) = mrYes)  then
          begin
            ThreadItem  :=  dl.ThreadItem;
            //
            //  ߋO (ŐV̔URL)
            //
            if  (FindKakolog(dl, ThreadItem.BBS.GetReadCgiUrl(ThreadItem))) then  begin
              //
              dl.FireEndDownload(False);
            end else begin
              //炸
              if  (ExtractHostName(ThreadItem.BBS.GetReadCgiUrl(ThreadItem)) <>
                   ExtractHostName(dl.OriginalUrl))  and
                  (dl.OriginalUrl <> '')  then
              begin
                //
                //  ߋO (URL)
                //
                if  (FindKakolog(dl, dl.OriginalUrl)) then  begin
                  //
                  dl.FireEndDownload(False);
                end else begin
                  //炸
                  if  (ZonuConfig.DolibUserName = '') then  begin
                    MessageDlg('ߋO͌t܂łB' + #13#10 +
                               'HTMLȌԂ܂B',
                               mtInformation, [mbOk], 0);
                  end else begin
                    MessageDlg('ߋO͌t܂łB', mtInformation, [mbOk], 0);
                  end;
                  Dispose(ParamData);
                  dl.FireEndDownload(True);
                end;
              end else begin
                //炸(VURL=URL)
                if  (ZonuConfig.DolibUserName = '') then  begin
                  MessageDlg('ߋO͌t܂łB' + #13#10 +
                             'HTMLȌԂ܂B',
                             mtInformation, [mbOk], 0);
                end else begin
                  MessageDlg('ߋO͌t܂łB', mtInformation, [mbOk], 0);
                end;
                Dispose(ParamData);
                dl.FireEndDownload(True);
              end;
            end;
          end else begin
            Dispose(ParamData);
            dl.FireEndDownload(False);
          end;
        end;
      end;
    end;
  end else begin
    //񒆂ȂICxg
    Dispose(ParamData);
    dl.FireEndDownload(False);
  end;
end;

//
//  ߋO
//
function TDolibScript.FindKakoLog(Downloader: TResDownloader;
  const Url: string): boolean;
var
  ThreadItem  : TThreadItem;
  inf         : TStringList;
  KakoUrl     : string;
  s : string;
begin
  Result      :=  False;
  ThreadItem  :=  Downloader.ThreadItem;
  //PParamData(Downloader.ParamData).FindKakoLogLevel := PParamData(Downloader.ParamData).FindKakoLogLevel + 1;
  inf :=  TStringList.Create;
  if  (FileExists(ChangeFileExt(ThreadItem.LocalDatFileName, '.inf')))  then  begin
    inf.LoadFromFile(ChangeFileExt(ThreadItem.LocalDatFileName, '.inf'));
  end;
  if  (Copy(Url, Length(Url) - 4, 5) <> '.html') then  begin
    if  (Length(ThreadItem.DatNo) > 9)  then  begin
      KakoUrl :=  'http://' +
                  ExtractHostName(Url) +  '/' +
                  ThreadItem.BoardID +
                  '/kako/'  +
                  Copy(ThreadItem.DatNo,  1,  4) + '/' +
                  Copy(ThreadItem.DatNo,  1,  5) + '/' +
                  ThreadItem.DatNo + '.dat';
    end else begin
      KakoUrl :=  'http://' +
                  ExtractHostName(Url) +  '/' +
                  ThreadItem.BoardID +
                  '/kako/'  +
                  Copy(ThreadItem.DatNo,  1,  3) + '/' +
                  ThreadItem.DatNo + '.dat';
    end;
    s :=  KakoUrl + '.gz';
  end else begin
    KakoUrl :=  Copy(Url, 1, Length(Url) - 5) + '.dat';
    s       :=  KakoUrl + '.gz';
  end;

  //
  //  dat.gz
  //
  if  (ExecDatDownload(Downloader, s, True, True)) then  begin
    //ߋOqɂŔ(.html/gz)
    Result  :=  True;
  end else begin
    if  (ExecDatDownload(Downloader, KakoUrl, False, True)) then  begin
      //ߋOqɂŔ(.html)
      Result  :=  True;
    end else begin
      //
      //  htmlO
      //
      if  (not DolibManager.Connected)  then  begin
        if  (ZonuConfig.DolibUserName <> '')  then  begin
          if  (not DolibManager.Connect)  then  begin
            DolibManager.Disconnect;
            //Adapter.FireResEndDownload(Self, False);
            //////////////////////////////Downloader.FireEndDownload(False);
          end;
        end;
      end;
      if  (ZonuConfig.DolibUserName <> '')  then  begin
        if  (DolibManager.Connected)  then  begin
          if  (ExecOfflaw(Downloader, KakoUrl)) then  begin
            Result  :=  True;
          end else begin
            //
          end;
        end;
      end;
      //ŌɏICxgR[ilXgĂꍇ͌Ă΂Ȃj
      //if  (PParamData(Downloader.ParamData).FindKakoLogLevel = 1) then  begin
        //Downloader.FireEndDownload(not Result);
      //end;
    end;
  end;
  inf.Free;
end;

//
//  DAT_E[h
//
function TDolibScript.ExecDatDownload(Downloader: TResDownloader;
  const DatUrl: string; const IsGz, KakoLog: boolean): boolean;
var
  ThreadItem  : TThreadItem;
  Res         : TRes;
  FZonuHttp   : TZonuHttp;
  ms          : TMemoryStream;
  Dat         : TStringList;
  inf         : TMemIniFile;
  Content     : string;
  ASubject    : string;
  StartIndex  : integer;
  i : integer;
begin
  Result      :=  True;
  ms          :=  TMemoryStream.Create;
  ThreadItem  :=  Downloader.ThreadItem;
  Dat         :=  TStringList.Create;
  if  (ThreadItem.DatExists)  then  begin
    Dat.LoadFromFile(ThreadItem.LocalDatFileName);
    StartIndex  :=  Dat.Count - 1;
  end else begin
    StartIndex  :=  -1;
  end;
  inf   :=  TMemIniFile.Create(ChangeFileExt(ThreadItem.LocalDatFileName, '.inf'));

  FZonuHttp             :=  TZonuHttp.Create(nil);
  FZonuHttp.Downloader  :=  Downloader;
  if  (AnsiPos('http://', DatUrl) = 1)  then  begin
    FZonuHttp.Url     :=  DatUrl;
  end else begin
    FZonuHttp.Url     :=  MonaUtils.MargeUrl(ExtractUrlPath(ThreadItem.BBS.GetReadCgiUrl(ThreadItem)), DatUrl);
  end;
  //FZonuHttp.CacheFileName   :=  ThreadItem.LocalDatFileName;
  FZonuHttp.InfSave         :=  False;  //.infɕۑȂ
  FZonuHttp.UserAgent   :=  DolibManager.UserAgent; //GetDolibUserAgent;
  try
    FZonuHttp.GetNotAsync(ms);
    Content   :=  Unzip(ms);
  except
    Content   :=  '';
  end;

  //if  (FZonuHttp.StatusCode <> 404) and (FZonuHttp.StatusCode <> 302)  then  begin
  if  (FZonuHttp.StatusCode > 199) and (FZonuHttp.StatusCode < 300)  then  begin
    Dat.Text  :=  Content;
    if  (ZonuEnviron.Cached)  then  begin
      inf.WriteInteger('LOG', 'SIZE', FZonuHttp.ContentLength);
      inf.WriteBool('LOG', 'KAKOLOG', KakoLog);
      inf.WriteString('LOG', 'LAST_MODIFIED', FZonuHttp.LastModified);
      inf.WriteString('LOG', 'ETAG', FZonuHttp.ETag);
      if  (not DirectoryExists(ExtractFilePath(ThreadItem.LocalDatFileName)))  then  begin
        ForceDirectories(ExtractFilePath(ThreadItem.LocalDatFileName));
      end;
      inf.UpdateFile;
      Dat.SaveToFile(ThreadItem.LocalDatFileName);
    end;
    ////////////////////////////////////////ThreadItem.IsKakoLog    :=  KakoLog;

    for i :=  StartIndex + 1 to  Dat.Count - 1 do  begin
      ThreadItem.DownloadCount := ThreadItem.DownloadCount + 1;
      Res       :=  TRes.Create;
      ParseDat(Dat[i], 0, Res, ASubject);
      Res.No    :=  Downloader.ResList.Count + 1;
      Res.BBS   :=  Downloader.BBS;
      Res.Site  :=  Downloader.Site;
      Res.ThreadTitle :=  ASubject;
      Downloader.FireDataEvt(Res, 0, 0);
      if  (ThreadItem.Title = '')  then  begin
        if  (ASubject <> '')  then  begin
          ThreadItem.Title :=  ASubject;
        end;
      end;
      Application.ProcessMessages;
    end;
    //Adapter.FireResEndDownload(Self, False);
    //////////////////////////////////////////*!*Downloader.FireEndDownload(False);
  end else begin
    Result  :=  False;
  end;
  FZonuHttp.Free;
  ms.Free;
  inf.Free;
  Dat.Free;
end;

//
//  offlaw.cgi HTML҂Xbhǂނł
//
function TDolibScript.ExecOfflaw(Downloader: TResDownloader;
  const Url: string = ''): boolean;
var
  ThreadItem  : TThreadItem;
  FZonuHttp : TZonuHttp;
  Size : integer;
  inf : TMemIniFile;
  x : string;
begin
  Result      :=  True;
  ThreadItem  :=  Downloader.ThreadItem;
  inf :=  TMemIniFile.Create(ChangeFileExt(ThreadItem.LocalDatFileName, '.inf'));
  Size  :=  inf.ReadInteger('LOG', 'SIZE', 0); //ݎ擾ς݂̃TCY
  if  (Size < 0)  then  Size := 0;
  inf.Free;
  if  (Url = '')  then  begin
    x :=  ThreadItem.BBS.GetReadCgiUrl(ThreadItem.Board, ThreadItem.DatNo);
    x :=  x + '&raw=.' +  IntToStr(Size);
  end else begin
    x  :=  Conv2chDatUrlToReadCgiURL(Url)  + '&raw=.' +  IntToStr(Size);
  end;
  x :=  StringReplace(x, 'read.cgi', 'offlaw.cgi', [rfIgnoreCase]);
  ThreadItem.RemoteDatFileName  :=  x;
  ThreadItem.LoadConfig;
  //擾Jn
  FZonuHttp                 :=  TZonuHttp.Create(nil);
  FZonuHttp.Downloader      :=  Downloader;
  FZonuHttp.OnZonuEnd       :=  EndOfflaw;
  //FZonuHttp.AutoPilotMode   :=  False;
  FZonuHttp.AcceptEncoding  :=  'gzip';
  FZonuHttp.UserAgent       :=  DolibManager.UserAgent;
  if  (DolibManager.Connected)  then  begin
    x :=  ThreadItem.RemoteDatFileName;
    x :=  StringReplace(x, '?bbs=', '/', [rfIgnoreCase]);
    x :=  StringReplace(x, '&key=', '/', [rfIgnoreCase]);
    x :=  StringReplace(x, '&raw=', '/?raw=', [rfIgnoreCase]);
    x :=  x + '&sid=' + DolibManager.SessionID; //FDolib.SessionID;
    ThreadItem.RemoteDatFileName  :=  x;
    FZonuHttp.NoCache         :=  False;
  end else begin
    //
    //  G[
    //
    Result  :=  False;
    Exit;
  end;
  FZonuHttp.Url :=  x;
  try
    FError  :=  False;
    FZonuHttp.GetCgiDiff(
                //ChangeFileExt(Downloader.ThreadItem.LocalDatFileName, '.inf'));
                ChangeFileExt(Downloader.ThreadItem.LocalDatFileName, '.inf'),
                False);
    if  (FError)  then  begin
      Result  :=  False;
    end else begin
    end;
  except
    Result  :=  False;
  end;
  //FZonuHttp.Free;
end;

//
//  offlaw.cgił̎擾I
//
procedure TDolibScript.EndOfflaw(Sender: TObject; Downloader: TDownloader;
  Error: boolean);
var
  ThreadItem    : TThreadItem;
  Res           : TRes;
  FZonuHttp     : TZonuHttp;
  inf           : TMemIniFile;
  ms            : TMemoryStream;
  Content       : string;
  ReadCgiState  : string;
  ReadCgiSize   : integer;
  i, Size, Max  : integer;
  Dat           : TStringList;
  StartIndex    : integer;
  ASubject      : string;
  Err: boolean;
begin
  inherited;
  FZonuHttp :=  TZonuHttp(Sender);
  ms  :=  TMemoryStream.Create;
  ms.LoadFromStream(FZonuHttp.RcvdStream);
  ///////////////////////////////////////////////////
  ExpandDat(ms, Content, ReadCgiSize, Max, ReadCgiState);
  ///////////////////////////////////////////////////
  ThreadItem  :=  TResDownloader(Downloader).ThreadItem;
  Dat :=  TStringList.Create;
  if  (ThreadItem.DatExists)  then  begin
    Dat.LoadFromFile(ThreadItem.LocalDatFileName);
  end;
  inf   :=  TMemIniFile.Create(ChangeFileExt(ThreadItem.LocalDatFileName, '.inf'));
  Size  :=  inf.ReadInteger('LOG', 'SIZE', 0);
  if  (Size < 0)  then  Size  :=  0;
  StartIndex  :=  Dat.Count - 1;

  ThreadItem.RemoteDatFileName  :=  GetDatUrl(ThreadItem);
  Err :=  False;

  if  (ReadCgiState = READCGI_OK)  or
      (ReadCgiState = READCGI_PARTIAL)  then
  begin
    //擾
    Size      :=  Size + ReadCgiSize;
    Dat.Text  :=  Dat.Text  + Content;
    Err := False;
  end else
  if  (AnsiPos('?', ReadCgiState) = 1)  then  begin
    //Not Found
    //inf.Free;
    //Dat.Free;
    //Exit;
    Err := True;
  end else
  if  (AnsiPos(READCGI_ERR, ReadCgiState) = 1)  then  begin
    Err := True;
    //Ȃ񂩃G[
    if  (ReadCgiState = READCGI_ERR_ABONE)  then  begin
      //"ځ[" 
      //FAboned :=  True;
      MessageDlg('Xrō폜ꂽ\܂B' + #13#10 +
                 'Ox폜Ď擾ȂĂB',
                 mtError, [mbok], 0);
    end else
    if  (ReadCgiState = READCGI_ERR_TIMEOUT)  then  begin
      //^CAEg
      MessageDlg(ReadCgiState,
                 mtError, [mbok], 0);
    end else
    if  (ReadCgiState = READCGI_ERR_CANTUSE)  then  begin
      //g܂
      MessageDlg(ReadCgiState,
                 mtError, [mbok], 0);
    end else begin
      //1001ȏ̓ǂݍ݂Ȃ
      FError  :=  True;
    end;
    //inf.Free;
    //Dat.Free;
    //Exit;
  end else
  if  (ReadCgiState = READCGI_INCR)  then  begin
    //(Incorrect)̏ꍇׂ͂Ẵf[^𑗐MB
    Size        :=  ReadCgiSize;
    //FClearLog :=  True;
    //FAboned   :=  True;
    Dat.Text    :=  Content;
    StartIndex  :=  0;
  end;

  if  (not Err) and (not Error) then  begin
    if  (ZonuEnviron.Cached)  then  begin
      inf.WriteInteger('LOG', 'SIZE', Size);
      inf.WriteBool('LOG', 'KAKOLOG', True);
      if  (not DirectoryExists(ExtractFilePath(ThreadItem.LocalDatFileName)))  then  begin
        ForceDirectories(ExtractFilePath(ThreadItem.LocalDatFileName));
      end;
      inf.UpdateFile;
      Dat.SaveToFile(ThreadItem.LocalDatFileName);
    end;
    //////////////////////////////////////////////ThreadItem.IsKakoLog    :=  True;

    for i :=  StartIndex + 1 to  Dat.Count - 1 do  begin
      ThreadItem.DownloadCount := ThreadItem.DownloadCount + 1;
      Res       :=  TRes.Create;
      ParseDat(Dat[i], 0, Res, ASubject);
      Res.No    :=  TResDownloader(Downloader).ResList.Count + 1;
      Res.BBS   :=  TResDownloader(Downloader).BBS;
      Res.Site  :=  TResDownloader(Downloader).Site;
      Res.ThreadTitle :=  ASubject;
      //FireResFound(Sender, Adapter, Res, 0, 0);
      TResDownloader(Downloader).FireDataEvt(Res, 0, 0);
      if  (ThreadItem.Title = '')  then  begin
        if  (ASubject <> '')  then  begin
          ThreadItem.Title :=  ASubject;
        end;
      end;
      Application.ProcessMessages;
    end;
  end;

  //Adapter.FireResEndDownload(Self, Error);
  {*!*TResDownloader(Downloader).FireEndDownload(Error);}
  inf.Free;
  Dat.Free;
end;

procedure TDolibScript.ExpandDat(ms: TMemoryStream; out Content: string;
  out Size, MaxSize: integer; out State: string);
var
  s : TStringList;
  er : string;
begin
  s :=  TStringList.Create;
  ms.Position :=  0;
  s.Text  :=  Unzip(ms);
  if  (s.Count > 0) then  begin
    //
    //  Xe[^X
    //
    if  (AnsiPos(READCGI_ERR, s.Text) = 1)  then  begin
      State := s[0];
      er  :=  s.Text;
      s.Free;
      Exit;
    end else
    if  (AnsiPos(READCGI_INCR, s.Text) = 1)  then  begin
      State := READCGI_INCR;
      Size  :=  StrToInt(Copy(s[0], Length(READCGI_INCR) + 1, AnsiPos('/', s[0]) - Length(READCGI_INCR) - 1));
      MaxSize :=  0;
      s.Delete(0);
    end else
    if  (AnsiPos(READCGI_OK, s.Text) = 1)  then begin
      State :=  READCGI_OK;
      Size  :=  StrToInt(Copy(s[0], Length(READCGI_OK) + 1, AnsiPos('/', s[0]) - Length(READCGI_OK) - 1));
      MaxSize :=  0;
      s.Delete(0);
    end else
    if  (AnsiPos(READCGI_PARTIAL, s.Text) = 1)  then  begin
      State :=  READCGI_OK;
      Size  :=  StrToInt(Copy(s[0], Length(READCGI_PARTIAL) + 1, AnsiPos('/', s[0]) - Length(READCGI_PARTIAL) - 1));
      MaxSize :=  0;
      s.Delete(0);
    end else begin
      State := '?';
      Size  :=  0;
      MaxSize  :=  0;
    end;
  end;
  Content :=  s.Text;
  s.Free;
end;

//
//  X
//
procedure TDolibScript.SendRes(Uploader: TResUploader);
var
  FZonuHttp : TZonuHttp;
  s : string;
begin
  FZonuHttp                 :=  TZonuHttp.Create(nil);
  FZonuHttp.Uploader        :=  Uploader;
  FZonuHttp.OnZonuSendEnd   :=  ZonuSendEnd;
  FZonuHttp.OnZonuSendError :=  ZonuSendError;
  FZonuHttp.Url             :=  GetSendUrl(Uploader.Board, Uploader.ThreadItem);
  FZonuHttp.UserAgent       :=  DolibManager.UserAgent;
  FZonuHttp.SendReferer     :=  Uploader.BBS.GetReadCgiUrl(Uploader.ThreadItem);
  FZonuHttp.SendContent     :=  Uploader.SendData.ConvSendContent(GetSendParam,
                                Uploader.ThreadItem.DatDate,
                                Uploader.ThreadItem.Profiler.MilliSec);
  s :=  ExtractHostName(FZonuHttp.Url);
  //29 begin
  {
  if  (ZonuConfig.CookieSPID[s] = '') then  begin
    FZonuHttp.SendCookie  :=  Uploader.SendData.ConvSendContent(GetSendCookie);
  end else begin
    //FZonuHttp.SendCookie  :=  'SPID=' + ZonuConfig.CookieSPID[s] + ';' +
    //                          Uploader.SendData.ConvSendContent(GetSendCookie);
    //PONΉ
    FZonuHttp.SendCookie  :=  ZonuConfig.CookieSPID[s] + ';' +
                              Uploader.SendData.ConvSendContent(GetSendCookie);
  end;
  }
  if  (ZonuConfig.Cookie2ch[s] = '') then  begin
    FZonuHttp.SendCookie  :=  Uploader.SendData.ConvSendContent(GetSendCookie);
  end else begin
    FZonuHttp.SendCookie  :=  ZonuConfig.Cookie2ch[s] + ';' +
                              Uploader.SendData.ConvSendContent(GetSendCookie) + ';' +
                              'hana=mogera';
  end;
  //29 end


  //11
  //BEΉ
  if  (Be2chManager.Connected)  then  begin
    if  (Be2chManager.IsBE(Uploader.Board)) or (ZonuConfig.BeSendLoginInfo) then //15
    begin
      if  (FZonuHttp.SendCookie <> '') then FZonuHttp.SendCookie := FZonuHttp.SendCookie + '; ';
      FZonuHttp.SendCookie  :=  FZonuHttp.SendCookie +  Be2chManager.GetCookie;
    end;
  end;

  if  (DolibManager.Connected)  then  begin
    FZonuHttp.SendContent :=  FZonuHttp.SendContent +  '&sid=' + DolibManager.SessionID; //FDolib.SessionID;
  end;
  FZonuHttp.Username        :=  Uploader.Board.BasicAuthorization_UserName; //8
  FZonuHttp.Password        :=  Uploader.Board.BasicAuthorization_Password; //8
  FZonuHttp.Send;
end;

//
//  ݏI
//
procedure TDolibScript.ZonuSendEnd(Sender: TObject; Uploader: TUploader;
  RecvStr: string; Error: boolean);
var
  FZonuHttp: TZonuHttp;
  TitleTag: string;
  s : string;
begin
  //11
  s     :=  RecvStr;
  Error :=  False;

  //12
  if  (TZonuHttp(Sender).Location = LOCATION_OOSUGI)  then  begin
    //Ɛl吙
    MessageDlg('Ɛl吙łB  ', mtError, [mbOk], 0);
    Error :=  True;
    inherited;
  end else
  if  (AnsiPos(BE2CH_WRITE_ERR, s) > 0)  then  begin
    //BEOC
    MessageDlg('BE@2chɃOCȂƏ߂܂B', mtError, [mbOk], 0);
    Error :=  True;
    inherited;
  end else begin
    TitleTag  :=  Copy(RecvStr, AnsiPos('<title>',RecvStr), Length(RecvStr));
    TitleTag  :=  Copy(TitleTag, 1, AnsiPos('</title>',TitleTag) + 7);
    {
    if  (AnsiPos(BBSCGI_ERR_SERVER_BUSY, RecvStr) > 0)  then
    begin
      MessageDlg('T[ôׂ̕ŏ߂܂B',
                 mtError, [mbOk], 0);
      inherited;
    end else
    }
    if  (AnsiPos(RES2CH_ERROR, RecvStr) > 0)  then  begin
      Uploader.FireUploadError(0, RecvStr);
    end else
    if  (AnsiPos('<title>NbL[mFI</title>', RecvStr) > 0)  or
        (AnsiPos('<title>NbL[mFI</title>', RecvStr) > 0)  or
        (AnsiPos('NbL[mF', TitleTag) > 0) or
        (AnsiPos(RES2CH_COOKIE, RecvStr) > 0)  then
    begin
      MessageDlg('NbL[Ȃ؂łI' + #13#10 +
                 'AVNȂ̂ĤׂŁAx߂ĂēeĂłBBB',
                 mtError, [mbOk], 0);
      FZonuHttp :=  TZonuHttp(Sender);
      S :=  FZonuHttp.RecvCookie;

      //29 begin (󂯎CookieSۑ)
      ZonuConfig.Cookie2ch[ExtractHostName(FZonuHttp.Url)] :=  S;
      //29 end

      //S :=  Copy(S, AnsiPos('SPID=', S) + 5, Length(S));
      //PONΉ
      S :=  Copy(S, 1, AnsiPos(';', S) - 1);
      ZonuConfig.CookieSPID[ExtractHostName(FZonuHttp.Url)] :=  S;
      //݃gC
      Uploader.FireSendRetry(RecvStr);
    end else
    if  (AnsiPos('<font size=+2 color=#FF0000>݃`FbNI </font>', RecvStr) > 0)  or
        (AnsiPos('<title> ݊mF܂ </title>', RecvStr) > 0)  or
        (AnsiPos('݊mF', TitleTag) > 0)  or
        (AnsiPos('="../test/subbbs.cgi">', RecvStr) > 0)  or
        (AnsiPos(RES2CH_FALSE, RecvStr) > 0)  then
    begin
      //
      //
      //
      if  (MessageDlg('݂ɊւĂ͗lXȃO񂪋L^Ă܂B' + #13#10 +
                      'eɊւĔӔC͑Sēe҂ɋA܂B' + #13#10 +
                      'ǑɔAlɖf鏑݂͍TĉB' + #13#10 +
                      'eꂽe̓Rs[EۑEpE]ړꍇ܂B' + #13#10 +
                      '' + #13#10 +
                      'SӔC𕉂Ƃď݂܂H',
                      mtWarning, [mbYes, mbNo], 0) = mrNo)  then
      begin
        //݃gC
        Uploader.FireSendRetry(RecvStr);
      end else begin
        //code̒o
        if  (AnsiPos('name=code value=', RecvStr) > 0)  then  begin
          S :=  Copy(RecvStr, AnsiPos('name=code value=', RecvStr) + 16, Length(RecvStr));
          Uploader.SendData.Crypt  :=  Copy(S, 1, AnsiPos('>', S) - 1);
        end else begin
          Uploader.SendData.Crypt  :=  '';
        end;
        //
        if  (Uploader is TThreadUploader) then  begin
          SendThread2(TThreadUploader(Uploader));
        end else
        if  (Uploader is TResUploader) then  begin
          SendRes2(TResUploader(Uploader));
        end;
      end;
    end else begin
      inherited;
    end;
  end;
end;

//
//  VX
//
procedure TDolibScript.SendThread(Uploader: TThreadUploader);
var
  FZonuHttp   : TZonuHttp;
  s: string;
begin
  FZonuHttp                 :=  TZonuHttp.Create(nil);
  FZonuHttp.Uploader        :=  Uploader;
  FZonuHttp.OnZonuSendEnd   :=  ZonuSendEnd;
  FZonuHttp.OnZonuSendError :=  ZonuSendError;
  FZonuHttp.Url             :=  GetNewThreadUrl(Uploader.Board);
  FZonuHttp.UserAgent       :=  DolibManager.UserAgent;
  FZonuHttp.SendReferer     :=  Uploader.Board.Url;
  FZonuHttp.SendContent     :=  Uploader.SendData.ConvSendContent(GetNewThreadParam,
                                                                  Uploader.ContentDate);
  {
  FZonuHttp.SendContent     :=  Uploader.SendData.ConvSendContent(GetSendParam,
                                Uploader.ThreadItem.DatDate,
                                Uploader.ThreadItem.Profiler.MilliSec);
  //FZonuHttp.SendCookie      :=  Uploader.SendData.ConvSendContent(GetNewThreadCookie);
  }
  s :=  ExtractHostName(FZonuHttp.Url);
  if  (ZonuConfig.CookieSPID[s] = '') then  begin
    FZonuHttp.SendCookie  :=  Uploader.SendData.ConvSendContent(GetSendCookie);
  end else begin
    //FZonuHttp.SendCookie  :=  'SPID=' + ZonuConfig.CookieSPID[s] + ';' +
    //                          Uploader.SendData.ConvSendContent(GetSendCookie);
    //PONΉ
    FZonuHttp.SendCookie  :=  ZonuConfig.CookieSPID[s] + ';' +
                              Uploader.SendData.ConvSendContent(GetSendCookie);
  end;
  //12
  //BEΉ
  if  (Be2chManager.IsBE(Uploader.Board)) and (Be2chManager.Connected) then  begin
    if  (FZonuHttp.SendCookie <> '') then FZonuHttp.SendCookie := FZonuHttp.SendCookie + '; ';
    FZonuHttp.SendCookie  :=  FZonuHttp.SendCookie +  Be2chManager.GetCookie;
  end;

  if  (DolibManager.Connected)  then  begin
    FZonuHttp.SendContent :=  FZonuHttp.SendContent +  '&sid=' + DolibManager.SessionID; //FDolib.SessionID;
  end;
  FZonuHttp.Username        :=  Uploader.Board.BasicAuthorization_UserName; //8
  FZonuHttp.Password        :=  Uploader.Board.BasicAuthorization_Password; //8
  FZonuHttp.Send;
end;

procedure TDolibScript.SendThread2(Uploader: TThreadUploader);
var
  FZonuHttp   : TZonuHttp;
  BBSini : TMemIniFile;
  Board : TBoard;
  s : string;
begin
  Board   :=  Uploader.Board;
  BBSini  :=  TMemIniFile.Create(Uploader.BBS.ConfigPath + INI_BBS);

  FZonuHttp                 :=  TZonuHttp.Create(nil);
  FZonuHttp.Uploader        :=  Uploader;
  FZonuHttp.OnZonuSendEnd   :=  ZonuSendEnd;
  FZonuHttp.OnZonuSendError :=  ZonuSendError;

  s :=  AnsiDequotedStr(BBSini.ReadString('NEWTHREADCGI', ExtractHostName(Board.Url), ''), '"');
  if  (s = '')  then  begin
    s :=  AnsiDequotedStr(BBSini.ReadString('NEWTHREADCGI', '*', ''), '"');
  end;
  s   :=  StringReplace(s, '<boardpath>', Board.BoardPath, [rfReplaceAll, rfIgnoreCase]);
  s   :=  StringReplace(s, '<host>', ExtractHostName(Board.Url), [rfReplaceAll, rfIgnoreCase]);
  FZonuHttp.Url             :=  StringReplace(s, 'bbs.cgi', 'subbbs.cgi', [rfIgnoreCase]);

  FZonuHttp.UserAgent       :=  DolibManager.UserAgent;
  FZonuHttp.SendReferer     :=  Uploader.Board.Url;

  s :=  AnsiDequotedStr(BBSini.ReadString('NEWTHREADPARAM', 'PARAM_SUB_SEND', ''), '"');
  s :=  Uploader.SendData.ConvSendContent(s, Uploader.ContentDate);



  FZonuHttp.SendContent     :=  s;
  //FZonuHttp.SendCookie      :=  Uploader.SendData.ConvSendContent(GetNewThreadCookie);
  s :=  ExtractHostName(FZonuHttp.Url);
  if  (ZonuConfig.CookieSPID[s] = '') then  begin
    FZonuHttp.SendCookie  :=  Uploader.SendData.ConvSendContent(GetSendCookie);
  end else begin
    //FZonuHttp.SendCookie  :=  'SPID=' + ZonuConfig.CookieSPID[s] + ';' +
    //                          Uploader.SendData.ConvSendContent(GetSendCookie);
    //PONΉ
    FZonuHttp.SendCookie  :=  ZonuConfig.CookieSPID[s] + ';' +
                              Uploader.SendData.ConvSendContent(GetSendCookie);
  end;
  if  (DolibManager.Connected)  then  begin
    FZonuHttp.SendContent :=  FZonuHttp.SendContent +  '&sid=' + DolibManager.SessionID; //FDolib.SessionID;
  end;

  FZonuHttp.OnZonuSendEnd   :=  ZonuSendEnd;
  FZonuHttp.OnZonuSendError :=  ZonuSendError;
  FZonuHttp.Send;

  BBSini.Free;
end;

function TDolibScript.GetReadCgiUrl(ThreadItem: TThreadItem): string;
var
  s: string;
begin
  s :=  inherited GetReadCgiUrl(ThreadItem);
  Result  :=  Conv2chNewUrl(s);
end;

function TDolibScript.GetRelativeLink: boolean;
begin
  Result  :=  False;
end;

procedure TDolibScript.SendRes2(Uploader: TResUploader);
var
  FZonuHttp   : TZonuHttp;
  BBSini : TMemIniFile;
  Board : TBoard;
  s : string;
begin
  Board   :=  Uploader.Board;
  BBSini  :=  TMemIniFile.Create(Uploader.BBS.ConfigPath + INI_BBS);

  FZonuHttp                 :=  TZonuHttp.Create(nil);
  FZonuHttp.Uploader        :=  Uploader;
  FZonuHttp.OnZonuSendEnd   :=  ZonuSendEnd;
  FZonuHttp.OnZonuSendError :=  ZonuSendError;

  s :=  AnsiDequotedStr(BBSini.ReadString('SENDCGI', ExtractHostName(Board.Url), ''), '"');
  if  (s = '')  then  begin
    s :=  AnsiDequotedStr(BBSini.ReadString('SENDCGI', '*', ''), '"');
  end;
  s   :=  StringReplace(s, '<boardpath>', Board.BoardPath, [rfReplaceAll, rfIgnoreCase]);
  s   :=  StringReplace(s, '<host>', ExtractHostName(Board.Url), [rfReplaceAll, rfIgnoreCase]);
  FZonuHttp.Url             :=  StringReplace(s, 'bbs.cgi', 'subbbs.cgi', [rfIgnoreCase]);

  FZonuHttp.UserAgent       :=  DolibManager.UserAgent;
  FZonuHttp.SendReferer     :=  Uploader.Board.Url;

  s :=  AnsiDequotedStr(BBSini.ReadString('SENDPARAM', 'PARAM_SUB_SEND', ''), '"');
  s :=  Uploader.SendData.ConvSendContent(s);



  FZonuHttp.SendContent     :=  s;
  //FZonuHttp.SendCookie      :=  Uploader.SendData.ConvSendContent(GetNewThreadCookie);
  s :=  ExtractHostName(FZonuHttp.Url);
  if  (ZonuConfig.CookieSPID[s] = '') then  begin
    FZonuHttp.SendCookie  :=  Uploader.SendData.ConvSendContent(GetSendCookie);
  end else begin
    //FZonuHttp.SendCookie  :=  'SPID=' + ZonuConfig.CookieSPID[s] + ';' +
    //                          Uploader.SendData.ConvSendContent(GetSendCookie);
    //PONΉ
    FZonuHttp.SendCookie  :=  ZonuConfig.CookieSPID[s] + ';' +
                              Uploader.SendData.ConvSendContent(GetSendCookie);
  end;
  if  (DolibManager.Connected)  then  begin
    FZonuHttp.SendContent :=  FZonuHttp.SendContent +  '&sid=' + DolibManager.SessionID; //FDolib.SessionID;
  end;

  FZonuHttp.OnZonuSendEnd   :=  ZonuSendEnd;
  FZonuHttp.OnZonuSendError :=  ZonuSendError;
  FZonuHttp.Send;

  BBSini.Free;
end;

procedure TDolibScript.WriteLog(const str: string);
begin
  FLog.Add(str);
  FLog.SaveToFile(ZonuConfig.UsersPath + 'hzblog.txt');
  if  (ZonuEnviron.LogStrings <> nil) then  begin
    ZonuEnviron.LogStrings.Add(' >' + str);
  end;
end;


destructor TDolibScript.Destory;
begin
  FLog.Free;
  inherited;
end;

constructor TDolibScript.Create(const ABBSID: string);
begin
  inherited;
  FLog  :=  TStringList.Create;
end;

function TDolibScript.GetResUrl(ThreadItem: TThreadItem; const StartNo,
  ToNo: integer; const NoFirst: boolean): string;
var
  s: string;
begin
  s :=  GetReadCgiUrl(ThreadItem);
  if  (StartNo = ToNo)  then  begin
    s :=  s + Format('%d', [StartNo]);
  end else begin
    s :=  s + Format('%d-%d', [StartNo, ToNo]);
  end;
  if  (NoFirst) then  begin
    s :=  s + 'n';
  end;
  Result  :=  s; 
end;

end.
